#ifndef CUDA_TIMER_HPP
#define CUDA_TIMER_HPP

#include <cassert>
#include <cuda_runtime.h>

class cuda_timer
{
public:
    cuda_timer()
        : start_event()
        , stop_event()
        , running(false)
        , elapsed_seconds(0.0)
    {
        cudaEventCreate(&start_event);
        cudaEventCreate(&stop_event);
    }

    ~cuda_timer ()
        {
            cudaEventDestroy(start_event);
            cudaEventDestroy(stop_event);
        }

    inline void start()
    {
        assert(!running);
        cudaEventRecord(start_event, 0);
        running = true;
    }

    inline void stop()
    {
        assert(running);
        cudaEventRecord(stop_event, 0);
        cudaEventSynchronize(stop_event);

        float phase_elapsed_ms = 0.0;
        cudaEventElapsedTime(&phase_elapsed_ms, start_event, stop_event);
        elapsed_seconds = elapsed_seconds + phase_elapsed_ms / 1e3;
        running = false;
    }

    inline double get_elapsed_seconds() const
    {
        assert (!running);
        return elapsed_seconds;
    }

private:
    cudaEvent_t start_event;
    cudaEvent_t stop_event;
    bool running;
    double elapsed_seconds;
};


#endif
