#ifndef FORWARD_DIFF_BENCHMARKS_HPP
#define FORWARD_DIFF_BENCHMARKS_HPP


#include "array_benchmark.hpp"
#include "test_array.hpp"
#include "random.hpp"

#include <cublas.h>
#include <cuda_runtime.h>


class gpu_shmem_forward_diff_benchmark : public array_benchmark
{
private:
    float_gen _generator;

    const gpu_float_array *_input;
    gpu_float_array *_result;

public:
    gpu_shmem_forward_diff_benchmark (
        const char * outfile_name,
        rand_src &rng
        ) : array_benchmark(outfile_name)
          , _generator(rng, -5, 5)
          , _input(NULL)
          , _result(NULL)
        {
        }

    virtual ~gpu_shmem_forward_diff_benchmark ()
        {
        }
    
    virtual void setup (const unsigned length)
        {
            _input = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _result = new gpu_float_array(length);
        }

    virtual void run ()
        {
            gpu_shmem_forward_diff(*_input, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _input;
        }

    virtual const char * get_name () const
        {
            return "gpu_shmem_forward_diff";
        }
};


class gpu_noshmem_forward_diff_benchmark : public array_benchmark
{
private:
    float_gen _generator;

    const gpu_float_array *_input;
    gpu_float_array *_result;

public:
    gpu_noshmem_forward_diff_benchmark (
        const char * outfile_name
        , rand_src &rng
        ) : array_benchmark(outfile_name)
          , _generator(rng, -5, 5)
          , _input(NULL)
          , _result(NULL)
        {
        }

    virtual ~gpu_noshmem_forward_diff_benchmark ()
        {
        }
    
    virtual void setup (const unsigned length)
        {
            _input = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _result = new gpu_float_array(length);
        }

    virtual void run ()
        {
            gpu_noshmem_forward_diff(*_input, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _input;
        }

    virtual const char * get_name () const
        {
            return "gpu_noshmem_forward_diff";
        }
};


#endif // FORWARD_DIFF_BENCHMARKS
