#ifndef   	RANDOM_HPP
#define   	RANDOM_HPP


#include <boost/random/mersenne_twister.hpp>
#include <boost/random/uniform_real.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/variate_generator.hpp>


template <class T>
struct rng_traits {};

template <>
struct rng_traits<int>
{
    typedef boost::uniform_int<int> distribution_t;
};

template <>
struct rng_traits<unsigned>
{
    typedef boost::uniform_int<unsigned> distribution_t;
};

template <>
struct rng_traits<float>
{
    typedef boost::uniform_real<float> distribution_t;
};


typedef boost::rand48 rand_src;


template <class T>
class rand_gen
{
private:
    boost::variate_generator<
        rand_src &,
        typename rng_traits<T>::distribution_t
        > _generator;

public:
    rand_gen<T>(rand_src &rng, T minval, T maxval)
        : _generator(rng,
                     typename rng_traits<T>::distribution_t(minval, maxval))
        {}

    inline T operator() ()
        {
            return _generator();
        }

    rand_src & get_rand_src () { return _generator.engine(); }
};


typedef rand_gen<float> float_gen;
typedef rand_gen<int> int_gen;
typedef rand_gen<unsigned> unsigned_gen;


#endif     /* !RANDOM_HPP */
