#ifndef RMSE_BENCHMARKS_HPP
#define RMSE_BENCHMARKS_HPP


#include "array_benchmark.hpp"
#include "test_array.hpp"
#include "random.hpp"
#include "gpu_rmse_benchmarks.hpp"

#include <cublas.h>
#include <cuda_runtime.h>


class gpu_rmse_fused_benchmark : public array_benchmark
{
private:
    float_gen _generator;

    const gpu_float_array *_x;
    const gpu_float_array *_y;
    float *_result;

public:
    gpu_rmse_fused_benchmark (const char * outfile_name, rand_src &rng)
        : array_benchmark(outfile_name)
        , _generator(rng, -5, 5)
        , _x(NULL)
        , _y(NULL)
        , _result(NULL)
        {
        }

    virtual ~gpu_rmse_fused_benchmark ()
        {
        }
    
    virtual void setup (const unsigned length)
        {
            _x = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _y = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _result = new float (0);
        }

    virtual void run ()
        {
            gpu_rmse(*_x, *_y, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _y;
            delete _x;
        }

    virtual const char * get_name () const
        {
            return "gpu_rmse_fused";
        }
};


class gpu_rmse_unfused_benchmark : public array_benchmark
{
private:
    float_gen _generator;

    const gpu_float_array *_x;
    const gpu_float_array *_y;
    float *_result;

public:
    gpu_rmse_unfused_benchmark (const char * outfile_name, rand_src &rng)
        : array_benchmark(outfile_name)
        , _generator(rng, -5, 5)
        , _x(NULL)
        , _y(NULL)
        , _result(NULL)
        {
        }

    virtual ~gpu_rmse_unfused_benchmark ()
        {
        }
    
    virtual void setup (const unsigned length)
        {
            _x = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _y = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _result = new float (0);
        }

    virtual void run ()
        {
            gpu_float_array subResult (_x->length());
            rmseSub(*_x, *_y, subResult);
            gpu_float_array powResult (_x->length());
            rmsePow(subResult, powResult);
            rmseSum(powResult, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _y;
            delete _x;
        }

    virtual const char * get_name () const
        {
            return "gpu_rmse_fused";
        }
};


#endif
