#ifndef SDOT_BENCHMARKS_HPP
#define SDOT_BENCHMARKS_HPP


#include "array_benchmark.hpp"
#include "test_array.hpp"
#include "random.hpp"

#include <cublas.h>
#include <cuda_runtime.h>


class cpu_sdot_benchmark : public array_benchmark
{
private:
    float_gen _generator;
    const float_array *_x;
    const float_array *_y;
    float *_result;

public:
    cpu_sdot_benchmark (const char * outfile_name, rand_src &rng)
        : array_benchmark(outfile_name)
        , _generator(rng, -5, 5)
        , _x(NULL)
        , _y(NULL)
        , _result(NULL)
        {
        }

    virtual ~cpu_sdot_benchmark ()
        {
        }
    
    virtual void setup (const unsigned length)
        {
            _x = new float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _y = new float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _result = new float (0);
        }

    virtual void run ()
        {
            cpu_sdot(*_x, *_y, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _y;
            delete _x;
        }

    virtual const char * get_name () const
        {
            return "cpu_sdot";
        }
};


class gpu_sdot_benchmark : public array_benchmark
{
private:
    float_gen _generator;

    const gpu_float_array *_x;
    const gpu_float_array *_y;
    float *_result;

public:
    gpu_sdot_benchmark (const char * outfile_name, rand_src &rng)
        : array_benchmark(outfile_name)
        , _generator(rng, -5, 5)
        , _x(NULL)
        , _y(NULL)
        , _result(NULL)
        {
        }

    virtual ~gpu_sdot_benchmark ()
        {
        }
    
    virtual void setup (const unsigned length)
        {
            _x = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _y = new gpu_float_array (
                length,
                new_test_array(_generator, length).get()
                );
            _result = new float (0);
        }

    virtual void run ()
        {
            gpu_sdot(*_x, *_y, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _y;
            delete _x;
        }

    virtual const char * get_name () const
        {
            return "gpu_sdot";
        }
};


class cublas_sdot_benchmark : public array_benchmark
{
private:
    float_gen _generator;

    const gpu_float_array *_x;
    gpu_float_array *_y;

public:
    cublas_sdot_benchmark (const char * outfile_name, rand_src &rng)
        : array_benchmark(outfile_name)
        , _generator(rng, -5, 5)
        , _x(NULL)
        , _y(NULL)
        {
            cublasInit();
        }

    virtual ~cublas_sdot_benchmark ()
        {
            cublasShutdown();
        }
    
    virtual void setup (const unsigned length)
        {
            _x = new gpu_float_array(
                length,
                new_test_array(_generator, length).get()
                );
            _y = new gpu_float_array(
                length,
                new_test_array(_generator, length).get()
                );
        }

    virtual void run ()
        {
            cublasSdot (
                _x->length(),
                _x->raw_array(),
                1,
                _y->raw_array(),
                1
                );
        }

    virtual void teardown ()
        {
            delete _y;
            delete _x;
        }

    virtual const char * get_name () const
        {
            return "cublas_sdot";
        }
};


#endif
