#ifndef TIMER_HPP
#define TIMER_HPP

#include <cassert>
#include <ctime>
#include <sys/time.h>

const clockid_t TIMER_CLOCK = CLOCK_PROCESS_CPUTIME_ID;

class timer
{
public:
    timer()
        : start_time()
        , running(false)
        , elapsed_seconds(0.0)
    {
    }

    inline void start()
    {
        assert(!running);
        clock_gettime(TIMER_CLOCK, &start_time);
        running = true;
    }

    inline void stop()
    {
        assert(running);
        elapsed_seconds += get_diff_seconds();
        running = false;
    }

    inline double get_elapsed_seconds() const
    {
        assert (!running);
        return elapsed_seconds;
    }

private:
    inline double get_diff_seconds() const
    {
        timespec t_time;
        clock_gettime(TIMER_CLOCK, &t_time);
        const double diff = (t_time.tv_sec - start_time.tv_sec)
            + 1e-9 * (t_time.tv_nsec - start_time.tv_nsec);
        assert(diff >= 0);
        return diff;
    }

    timespec start_time;
    bool running;
    double elapsed_seconds;
};


#endif
