#ifndef BARRACUDA_ARRAY_HPP
#define BARRACUDA_ARRAY_HPP

#include "host_and_device.hpp"
#include "utility_functions.hpp"

#ifndef NO_CUDA
#include <cuda_runtime.h>
#endif

#include <cassert>
#include <algorithm>

#ifndef NO_CUDA
template <class T>
class gpu_array;
#endif

template <class T>
class array
{
public:
  // -----------------------------------------------------------
  // Constructors / Destructor
  // -----------------------------------------------------------
  array()
    : _length(0)
    , _store(new T[_length])
  {
  }

  array(int length)
    : _length(length)
    , _store(new T[_length])
  {
  }

  array(int length, const T *values)
    : _length(length)
    , _store(new T[_length])
  {
    std::copy(values, values + _length, _store);
  }

#ifndef NO_CUDA
  array(const gpu_array<T> &other)
    : _length(other.length())
    , _store(new T[_length])
  {
    cudaMemcpy(_store,
               other.raw_array(),
               _length * sizeof(T),
               cudaMemcpyDeviceToHost);
  }
#endif

  ~array()
  {
    delete [] _store;
  }

  // -----------------------------------------------------------
  // Indexing
  // -----------------------------------------------------------
  const T & operator [](int idx) const
  {
    assert(idx < _length);
    return _store[idx];
  }

  T & operator [] (int idx)
  {
    assert(idx < _length);
    return _store[idx];
  }

  // -----------------------------------------------------------
  // Accessors
  // -----------------------------------------------------------
  int length () const
  {
    return _length;
  }

private:
  array<T> & operator = (const array<T> &);
  array(const array<T> &);

private:
  const int _length;
  T *_store;
};

#endif
