#ifndef ARRAY_VIEW_HPP
#define ARRAY_VIEW_HPP

template <class T>
class array_view
{
public:
    inline
    array_view (
        array<T> &arr
        ) : _array(arr)
          , _slice_info(0, arr.length(), 1)
        {
        }

    inline
    array_view (
        array<T> &arr,
        const slice slice_info
        ) : _array(arr)
          , _slice_info(slice_info)
        {
            assert(_slice_info.stride() != 0);
            assert(_slice_info.start() <= arr.length());
            assert(_slice_info.stop() <= arr.length());
        }

    HOST_AND_DEVICE
    inline
    int length () const
    {
        return _slice_info.length();
    }

    // -----------------------------------------------------------
    // Indexing
    // -----------------------------------------------------------
    HOST_AND_DEVICE
    inline
    const T & operator [](int idx) const
    {
        assert(idx < length());
        return _array[_slice_info(idx)];
    }

    HOST_AND_DEVICE
    inline
    T & operator [] (int idx)
    {
        assert(idx < length());
        return _array[_slice_info(idx)];
    }

private:
    array<T> &_array;
    const slice _slice_info;
};

#endif // ARRAY_VIEW_HPP
