#ifndef BARRACUDA_GPU_MATRIX_HPP
#define BARRACUDA_GPU_MATRIX_HPP

#include <cuda_runtime.h>

#include <cassert>
#include <algorithm>

#include "host_and_device.hpp"
#include "utility_functions.hpp"

template <class T>
class gpu_matrix
{
public:
  // -----------------------------------------------------------
  // Constructors / Destructor
  // -----------------------------------------------------------
  gpu_matrix(const int rows, const int cols)
    : _rows(rows)
    , _cols(cols)
  {
    const size_t width = _cols * sizeof(T);
    const size_t height = _rows;
    cudaMallocPitch((void **)&_store, &_pitch, width, height);
  }

  gpu_matrix(const int rows, const int cols, const T *values)
    : _rows(rows)
    , _cols(cols)
  {
    const size_t width = _cols * sizeof(T);
    const size_t height = _rows;
    const size_t source_pitch = width;
    cudaMallocPitch((void **)&_store, &_pitch, width, height);
    cudaMemcpy2D(_store,
                 _pitch,
                 values,
                 source_pitch,
                 width,
                 height,
                 cudaMemcpyHostToDevice);
  }

  ~gpu_matrix()
  {
    cudaFree(_store);
  }

  // -----------------------------------------------------------
  // Accessors
  // -----------------------------------------------------------
  int rows() const
  {
    return _rows;
  }

  int cols() const
  {
    return _cols;
  }

  int pitch() const
  {
    return _pitch;
  }

  const T * raw_matrix() const
  {
    return _store;
  }

  T * raw_matrix()
  {
    return _store;
  }

private:
  const size_t _rows;
  const size_t _cols;
  size_t _pitch;
  T *_store;
};

#endif
