#ifndef BARRACUDA_KERNEL_MATRIX_HPP
#define BARRACUDA_KERNEL_MATRIX_HPP

#include "gpu_matrix.hpp"

#include "host_and_device.hpp"
#include "utility_functions.hpp"

template <class T>
class kernel_matrix
{
public:
    // -----------------------------------------------------
    // Constructors / Destructor
    // -----------------------------------------------------
    inline
    kernel_matrix ( const gpu_matrix<T> &arr )
        : _rows  (arr.rows())
        , _cols  (arr.cols())
        , _pitch (arr.pitch())
        , _store (const_cast<T *>(arr.raw_matrix()))
    {
    }

    // -----------------------------------------------------
    // Indexing
    // -----------------------------------------------------
    inline
    __device__
    T operator () (int row, int col) const
    {
        return ((T *) ((char *) _store + row * _pitch))[col];
    }
  
    inline
    __device__
    T & operator () (int row, int col)
    {
        return ((T *) ((char *) _store + row * _pitch))[col];
    }
  
    // -----------------------------------------------------
    // Accessors
    // -----------------------------------------------------
    inline
    HOST_AND_DEVICE
    int rows () const
    {
        return _rows;
    }

    inline
    HOST_AND_DEVICE
    int cols () const
    {
        return _cols;
    }

private:  
    int _rows;
    int _cols;
    int _pitch;
    T *_store;
};

#endif
