#ifndef KERNEL_MATRIX_VIEW_HPP
#define KERNEL_MATRIX_VIEW_HPP

#include "host_and_device.hpp"
#include "utility_functions.hpp"
#include "kernel_matrix.hpp"
#include "gpu_matrix.hpp"
#include "gpu_matrix_view.hpp"

template <class T>
class kernel_matrix_view
{
public:
    inline
    kernel_matrix_view (
        kernel_matrix<T> mat
        ) : _matrix(mat)
          , _row_slice(0, mat.rows(), 1)
          , _col_slice(0, mat.cols(), 1)
        {
        }

    inline
    kernel_matrix_view (
        kernel_matrix<T> mat,
        const slice row_slice,
        const slice col_slice
        ) : _matrix(mat)
          , _row_slice(row_slice)
          , _col_slice(col_slice)
        {
            assert(_row_slice.stride() != 0);
            assert(_row_slice.start() <= mat.rows());
            assert(_row_slice.stop() <= mat.rows());
            assert(_col_slice.stride() != 0);
            assert(_col_slice.start() <= mat.cols());
            assert(_col_slice.stop() <= mat.cols());
        }

    inline
    kernel_matrix_view ( gpu_matrix_view<T> &view )
        : _matrix(view._matrix)
        , _row_slice(view._row_slice)
        , _col_slice(view._row_slice)
        {
        }

    HOST_AND_DEVICE
    inline
    int rows () const
    {
        return _row_slice.length();
    }

    HOST_AND_DEVICE
    inline
    int cols () const
    {
        return _col_slice.length();
    }

    // -----------------------------------------------------------
    // Indexing
    // -----------------------------------------------------------
    DEVICE
    inline
    const T & operator ()(int row, int col) const
    {
        return _matrix(_row_slice(row), _col_slice(col));
    }

    DEVICE
    inline
    T & operator ()(int row, int col)
    {
        return _matrix(_row_slice(row), _col_slice(col));
    }

private:
    kernel_matrix<T> _matrix;
    const slice _row_slice;
    const slice _col_slice;
};

#endif // KERNEL_MATRIX_VIEW_HPP
