#ifndef MATRIX_VIEW_HPP
#define MATRIX_VIEW_HPP

template <class T>
class matrix_view
{
public:
    inline
    matrix_view (
        matrix<T> &mat
        ) : _matrix(mat)
          , _row_slice(0, mat.rows(), 1)
          , _col_slice(0, mat.cols(), 1)
        {
        }

    inline
    matrix_view (
        matrix<T> &mat,
        const slice row_slice,
        const slice col_slice
        ) : _matrix(mat)
          , _row_slice(row_slice)
          , _col_slice(col_slice)
        {
            assert(_row_slice.stride() != 0);
            assert(_row_slice.start() <= mat.rows());
            assert(_row_slice.stop() <= mat.rows());
            assert(_col_slice.stride() != 0);
            assert(_col_slice.start() <= mat.cols());
            assert(_col_slice.stop() <= mat.cols());
        }

    HOST_AND_DEVICE
    inline
    int rows () const
    {
        return _row_slice.length();
    }

    HOST_AND_DEVICE
    inline
    int cols () const
    {
        return _col_slice.length();
    }

    // -----------------------------------------------------------
    // Indexing
    // -----------------------------------------------------------
    HOST_AND_DEVICE
    inline
    const T & operator ()(int row, int col) const
    {
        assert(row < rows());
        assert(col < cols());
        return _matrix(_row_slice(row), _col_slice(col));
    }

    HOST_AND_DEVICE
    inline
    T & operator ()(int row, int col)
    {
        assert(row < rows());
        assert(col < cols());
        return _matrix(_row_slice(row), _col_slice(col));
    }

private:
    matrix<T> &_matrix;
    const slice _row_slice;
    const slice _col_slice;
};

#endif // MATRIX_VIEW_HPP
