#include "gpu_black_scholes.hpp"
#include "cpu_black_scholes.hpp"
#include "barracuda_test.hpp"

bool test_black_scholes (float_gen &generator, const unsigned len)
{
    float_test_array sVals = new_test_array(generator, len, 10.0f, 300.0f);
    float_test_array xVals = new_test_array(generator, len, 1.0f, 300.0f);
    float_test_array tVals = new_test_array(generator, len, 0.25f, 10.0f);

    const gpu_float_array gpu_s(len, sVals.get());
    const gpu_float_array gpu_x(len, xVals.get());
    const gpu_float_array gpu_t(len, tVals.get());
    gpu_float_array gpu_result(len);

    const float_array cpu_s(len, sVals.get());
    const float_array cpu_x(len, xVals.get());
    const float_array cpu_t(len, tVals.get());
    float_array cpu_result(len);

    const float r = 0.02f;
    const float v = 0.30f;

    gpu_black_scholes(gpu_s, gpu_x, gpu_t, r, v, gpu_result);
    cpu_black_scholes(cpu_s, cpu_x, cpu_t, r, v, cpu_result);

    return compare_arrays(gpu_result, cpu_result);
}
