#include "gpu_noshmem_jacobi_stencil.hpp"
#include "gpu_shmem_jacobi_stencil.hpp"
#include "cpu_jacobi_stencil.hpp"
#include "barracuda_test.hpp"


bool test_jacobi_stencil (float_gen &generator,
                          const unsigned rows,
                          const unsigned cols)
{
  if (rows < 2 || cols < 2) return true;

  const float_test_array initVals1 = new_test_array(generator, rows * cols);

  const gpu_float_matrix gpu_noshmem_input(rows, cols, initVals1.get());
  gpu_float_matrix gpu_noshmem_result(rows, cols, initVals1.get());

  const gpu_float_matrix gpu_shmem_input(rows, cols, initVals1.get());
  gpu_float_matrix gpu_shmem_result(rows, cols, initVals1.get());

  const float_matrix cpu_input(rows, cols, initVals1.get());
  float_matrix cpu_result(rows, cols, initVals1.get());

  const slice row_slice (1, rows - 1, 1);
  const slice col_slice (1, cols - 1, 1);

  gpu_noshmem_jacobi_stencil (
      gpu_noshmem_input,
      gpu_float_matrix_view(gpu_noshmem_result, row_slice, col_slice)
      );
  gpu_shmem_jacobi_stencil (
      gpu_shmem_input,
      gpu_float_matrix_view(gpu_shmem_result, row_slice, col_slice)
      );
  cpu_jacobi_stencil (
      cpu_input,
      float_matrix_view(cpu_result, row_slice, col_slice)
      );

  return
      compare_matrices(float_matrix(gpu_noshmem_result), cpu_result) &&
      compare_matrices(float_matrix(gpu_shmem_result), cpu_result);
}
