module Main (main) where

import Language.Barracuda

-- | One step of Jacobi iteration
jacobiStep :: MExpr Float -> MExpr Float
jacobiStep m = mmap (/ 4) (n .+. s .+. e .+. w)
  where
    (.+.) = mzipWith (+)
    n = mslice m (0, mrows m - 2, 1) (1, mcols m - 1, 1)
    s = mslice m (2, mrows m, 1)     (1, mcols m - 1, 1)
    e = mslice m (1, mrows m - 1, 1) (2, mcols m, 1)
    w = mslice m (1, mrows m - 1, 1) (0, mcols m - 2, 1)

fun :: String -> [Function]
fun p = [Function jacobiStep (p ++ "_jacobi_stencil") ["input"] "result"]

main :: IO ()
main = do
  compile
    defaultCppConfig
    ("cpu_jacobi_stencil.hpp", "cpu_jacobi_stencil.cpp")
    (fun "cpu")
  compile
    defaultCudaConfig
    ("gpu_noshmem_jacobi_stencil.hpp", "gpu_noshmem_jacobi_stencil.cu")
    (fun "gpu_noshmem")
  compile
    (CudaTarget CudaConfig { sharedMemoryOptimization = True })
    ("gpu_shmem_jacobi_stencil.hpp", "gpu_shmem_jacobi_stencil.cu")
    (fun "gpu_shmem")
