#ifndef   	RANDOM_HPP
#define   	RANDOM_HPP


#include <boost/random/mersenne_twister.hpp>
#include <boost/random/uniform_real.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/variate_generator.hpp>


template <class T>
struct rng_traits {};

template <>
struct rng_traits<int>
{
  typedef boost::uniform_int<int> distribution_t;
};

template <>
struct rng_traits<unsigned>
{
  typedef boost::uniform_int<unsigned> distribution_t;
};

template <>
struct rng_traits<float>
{
  typedef boost::uniform_real<float> distribution_t;
};


template <class T>
class rand_gen
{
public:
    typedef boost::mt19937 base_generator_t;

private:
    base_generator_t _rng;
    boost::variate_generator<base_generator_t &,
                             typename rng_traits<T>::distribution_t> _generator;

public:
    rand_gen<T>(T minval, T maxval)
        : _rng()
        , _generator(_rng, typename rng_traits<T>::distribution_t(minval, maxval))
        {}

    inline T operator() ()
        {
            return _generator();
        }

    base_generator_t & engine() { return _generator.engine(); }
};


typedef rand_gen<float> float_gen;
typedef rand_gen<int> int_gen;
typedef rand_gen<unsigned> unsigned_gen;


#endif 	    /* !RANDOM_HPP */
